package sun.security.x509;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import sun.security.util.ObjectIdentifier;
import sun.security.util.DerOutputStream;
import java.security.cert.CertificateException;
import java.io.OutputStream;
import java.lang.reflect.InvocationTargetException;
import sun.misc.HexDumpEncoder;
import sun.security.util.DerValue;
import java.io.IOException;
import sun.security.util.DerInputStream;
import java.util.Collections;
import java.util.TreeMap;
import java.util.Map;
import sun.security.util.Debug;

public class CertificateExtensions implements CertAttrSet<Extension>
{
    public static final String IDENT = "x509.info.extensions";
    public static final String NAME = "extensions";
    private static final Debug debug;
    private Map<String, Extension> map;
    private boolean unsupportedCritExt;
    private Map<String, Extension> unparseableExtensions;
    private static Class[] PARAMS;
    
    public CertificateExtensions() {
        this.map = Collections.synchronizedMap(new TreeMap());
        this.unsupportedCritExt = false;
    }
    
    public CertificateExtensions(DerInputStream derInputStream) throws IOException {
        this.map = Collections.synchronizedMap(new TreeMap());
        this.unsupportedCritExt = false;
        this.init(derInputStream);
    }
    
    private void init(DerInputStream derInputStream) throws IOException {
        DerValue[] sequence = derInputStream.getSequence(5);
        for (int i = 0; i < sequence.length; ++i) {
            this.parseExtension(new Extension(sequence[i]));
        }
    }
    
    private void parseExtension(Extension obj) throws IOException {
        try {
            Class<Object> class1 = OIDMap.getClass(obj.getExtensionId());
            if (class1 == null) {
                if (obj.isCritical()) {
                    this.unsupportedCritExt = true;
                }
                if (this.map.put(obj.getExtensionId().toString(), obj) == null) {
                    return;
                }
                throw new IOException("Duplicate extensions not allowed");
            }
            else {
                CertAttrSet<Object> set = (CertAttrSet)class1.getConstructor(CertificateExtensions.PARAMS).newInstance(new Object[] { Boolean.valueOf(obj.isCritical()), obj.getExtensionValue() });
                if (this.map.put(set.getName(), (Extension)set) != null) {
                    throw new IOException("Duplicate extensions not allowed");
                }
            }
        }
        catch (InvocationTargetException ex) {
            Throwable targetException = ex.getTargetException();
            if (!obj.isCritical()) {
                if (this.unparseableExtensions == null) {
                    this.unparseableExtensions = new TreeMap();
                }
                this.unparseableExtensions.put(obj.getExtensionId().toString(), new UnparseableExtension(obj, targetException));
                if (CertificateExtensions.debug != null) {
                    CertificateExtensions.debug.println("Debug info only. Error parsing extension: " + obj);
                    targetException.printStackTrace();
                    System.err.println(new HexDumpEncoder().encodeBuffer(obj.getExtensionValue()));
                }
                return;
            }
            else {
                if ((targetException instanceof IOException)) {
                    throw (IOException)targetException;
                }
                throw new IOException(targetException);
            }
        }
        catch (IOException ex2) {
            throw ex2;
        }
        catch (Exception cause) {
            throw new IOException(cause);
        }
    }
    
    @Override
    public void encode(OutputStream outputStream) throws CertificateException, IOException {
        this.encode(outputStream, false);
    }
    
    public void encode(OutputStream outputStream, boolean b) throws CertificateException, IOException {
        DerOutputStream derOutputStream = new DerOutputStream();
        Object[] array = this.map.values().toArray();
        for (int i = 0; i < array.length; ++i) {
            if ((array[i] instanceof CertAttrSet)) {
                ((CertAttrSet)array[i]).encode(derOutputStream);
            }
            else if ((array[i] instanceof Extension)) {
                ((Extension)array[i]).encode(derOutputStream);
            }
            else {
                throw new CertificateException("Illegal extension object");
            }
        }
        DerOutputStream derOutputStream2 = new DerOutputStream();
        derOutputStream2.write((byte)48, derOutputStream);
        DerOutputStream derOutputStream3;
        if (!b) {
            derOutputStream3 = new DerOutputStream();
            derOutputStream3.write(DerValue.createTag((byte)(-128), true, (byte)3), derOutputStream2);
        }
        else {
            derOutputStream3 = derOutputStream2;
        }
        outputStream.write(derOutputStream3.toByteArray());
    }
    
    @Override
    public void set(String s, Object o) throws IOException {
        if ((o instanceof Extension)) {
            this.map.put(s, (Extension)o);
            return;
        }
        throw new IOException("Unknown extension type.");
    }
    
    @Override
    public Extension get(String str) throws IOException {
        Extension extension = (Extension)this.map.get(str);
        if (extension == null) {
            throw new IOException("No extension found with name " + str);
        }
        return extension;
    }
    
    Extension getExtension(String s) {
        return (Extension)this.map.get(s);
    }
    
    @Override
    public void delete(String str) throws IOException {
        if (this.map.get(str) == null) {
            throw new IOException("No extension found with name " + str);
        }
        this.map.remove(str);
        return;
    }
    
    public String getNameByOid(ObjectIdentifier obj) throws IOException {
        for (String s : this.map.keySet()) {
            if (((Extension)this.map.get(s)).getExtensionId().equals(obj)) {
                return s;
            }
            continue;
        }
        return null;
    }
    
    @Override
    public Enumeration<Extension> getElements() {
        return Collections.enumeration(this.map.values());
    }
    
    public Collection<Extension> getAllExtensions() {
        return this.map.values();
    }
    
    public Map<String, Extension> getUnparseableExtensions() {
        if (this.unparseableExtensions == null) {
            return Collections.emptyMap();
        }
        return this.unparseableExtensions;
    }
    
    @Override
    public String getName() {
        return "extensions";
    }
    
    public boolean hasUnsupportedCriticalExtension() {
        return this.unsupportedCritExt;
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof CertificateExtensions)) {
            return false;
        }
        Object[] array = ((CertificateExtensions)o).getAllExtensions().toArray();
        int length = array.length;
        if (length != this.map.size()) {
            return false;
        }
        String s = null;
        int i = 0;
        while (i < length) {
            if ((array[i] instanceof CertAttrSet)) {
                s = ((CertAttrSet)array[i]).getName();
            }
            Extension extension = (Extension)array[i];
            if (s == null) {
                s = extension.getExtensionId().toString();
            }
            Extension extension2 = (Extension)this.map.get(s);
            if (extension2 == null) {
                return false;
            }
            if (!extension2.equals(extension)) {
                return false;
            }
            ++i;
        }
        return this.getUnparseableExtensions().equals(((CertificateExtensions)o).getUnparseableExtensions());
    }
    
    @Override
    public int hashCode() {
        return this.map.hashCode() + this.getUnparseableExtensions().hashCode();
    }
    
    @Override
    public String toString() {
        return this.map.toString();
    }
    
    static {
        debug = Debug.getInstance("x509");
        CertificateExtensions.PARAMS = new Class[] { Boolean.class, Object.class };
    }
}
